@@ -6,9 +6,9 @@ from itertools import chain
 
 import django
 from django.db.models.aggregates import Avg, Count, StdDev, Variance
-from django.db.models.expressions import Ref, Subquery, Value
+from django.db.models.expressions import Ref, Subquery, Value, Window
 from django.db.models.functions import (
-    Chr, ConcatPair, Greatest, Least, Length, LPad, Repeat, RPad, StrIndex, Substr, Trim
+    Chr, ConcatPair, Greatest, Least, Length, LPad, Random, Repeat, RPad, StrIndex, Substr, Trim
 )
 from django.db.models.sql import compiler
 from django.db.transaction import TransactionManagementError
@@ -129,6 +129,41 @@ def _as_sql_variance(self, compiler, connection):
         function = '%sP' % function
     return self.as_sql(compiler, connection, function=function)
 
+def _as_sql_window(self, compiler, connection, template=None):
+    connection.ops.check_expression_support(self)
+    if not connection.features.supports_over_clause:
+        raise NotSupportedError("This backend does not support window expressions.")
+    expr_sql, params = compiler.compile(self.source_expression)
+    window_sql, window_params = [], ()
+
+    if self.partition_by is not None:
+        sql_expr, sql_params = self.partition_by.as_sql(
+            compiler=compiler,
+            connection=connection,
+            template="PARTITION BY %(expressions)s",
+        )
+        window_sql.append(sql_expr)
+        window_params += tuple(sql_params)
+
+    if self.order_by is not None:
+        order_sql, order_params = compiler.compile(self.order_by)
+        window_sql.append(order_sql)
+        window_params += tuple(order_params)
+    else:
+        # MSSQL window functions require an OVER clause with ORDER BY
+        window_sql.append('ORDER BY (SELECT NULL)')
+    
+    if self.frame:
+        frame_sql, frame_params = compiler.compile(self.frame)
+        window_sql.append(frame_sql)
+        window_params += tuple(frame_params)
+
+    template = template or self.template
+
+    return (
+        template % {"expression": expr_sql, "window": " ".join(window_sql).strip()},
+        (*params, *window_params),
+    )
 
 def _cursor_iter(cursor, sentinel, col_count, itersize):
     """
@@ -281,7 +316,9 @@ class SQLCompiler(compiler.SQLCompiler):
                 if for_update_part and self.connection.features.for_update_after_from:
                     from_.insert(1, for_update_part)
 
-                result += [', '.join(out_cols), 'FROM', *from_]
+                result += [', '.join(out_cols)]
+                if from_:
+                    result += ['FROM', *from_]
                 params.extend(f_params)
 
                 if where:
@@ -311,7 +348,13 @@ class SQLCompiler(compiler.SQLCompiler):
 
             if order_by:
                 ordering = []
-                for _, (o_sql, o_params, _) in order_by:
+                for expr, (o_sql, o_params, _) in order_by:
+                    if expr:
+                        src = next(iter(expr.get_source_expressions()))
+                        if isinstance(src, Random):
+                            # ORDER BY RAND() doesn't return rows in random order
+                            # replace it with NEWID()
+                            o_sql = o_sql.replace('RAND()', 'NEWID()')
                     ordering.append(o_sql)
                     params.extend(o_params)
                 result.append('ORDER BY %s' % ', '.join(ordering))
@@ -416,6 +459,9 @@ class SQLCompiler(compiler.SQLCompiler):
         if django.VERSION >= (3, 1):
             if isinstance(node, json_KeyTransform):
                 as_microsoft = _as_sql_json_keytransform
+        if django.VERSION >= (4, 1):
+            if isinstance(node, Window):
+                as_microsoft = _as_sql_window
         if as_microsoft:
             node = node.copy()
             node.as_microsoft = types.MethodType(as_microsoft, node)
